package com.example.model;

import javafx.geometry.Point2D;
import javafx.geometry.Dimension2D;
import javafx.scene.effect.Light;

import java.util.Random;

public class Ball {

/* Pixels per second */

    private final double velocity;

    private final double radius;
    private int angle;

    private final Dimension2D panelSize;

    private Point2D centerPoint;

    private final Random random;

    private long lastTime = 0;

    public Ball(Point2D centerPoint, double radius, Dimension2D panelSize) {
        this.centerPoint = centerPoint;
        this.radius = radius;
        this.panelSize = panelSize;
        this.velocity = 150.0;
        this.random = new Random();
        setRandomAngle();
    }

    public void reverseYDirection(long time) {
        angle = bounceYAngle(angle);
        angle += (angle < 0) ? 360 : 0;
        moveBall(time);
    }

    public void moveBall(long time) {
        if (lastTime == 0 || lastTime == time) {
            lastTime = time;
            return;
        }

        double theta = Math.toRadians(angle);
        double distance = velocity * (time - lastTime) / 1.e9;
        centerPoint = centerPoint.add(Math.cos(theta) * distance, Math.sin(theta) * distance);

        if (centerPoint.getY() < radius) {
            double adjustment = radius - centerPoint.getY();
            centerPoint = centerPoint.add(0, adjustment);
            angle = bounceYAngle(angle);
        }

        if ((centerPoint.getY() + radius) > panelSize.getHeight()) {
            double adjustment = panelSize.getHeight() -
                    centerPoint.getY()  - radius;
            centerPoint = centerPoint.add(0, adjustment);
            angle = bounceYAngle(angle);
        }

        if (centerPoint.getX() < radius) {
            double adjustment = radius - centerPoint.getX();
            centerPoint = centerPoint.add(adjustment, 0);
            angle = bounceXAngle(angle);
        }

        if ((centerPoint.getX() + radius) > panelSize.getWidth()) {
            double adjustment = panelSize.getWidth() -
                    centerPoint.getX() - radius;
            centerPoint = centerPoint.add(adjustment, 0);
            angle = bounceXAngle(angle);
        }

        lastTime = time;
    }

    private int bounceXAngle(int angle) {
        return 180 - angle;
    }


    private int bounceYAngle(int angle) {
        return 360 - angle;
    }

    public Point2D getCenterPoint() {
        return centerPoint;
    }

    public void setCenterPoint(Point2D centerPoint) {
        this.centerPoint = centerPoint;
        setRandomAngle();
    }

    private void setRandomAngle() {
        int[] angles = {30, 150, 210, 330 };
        this.angle = angles[random.nextInt(angles.length)];
    }

    public double getRadius() {
        return radius;
    }
}
