package com.example.model;

import com.example.poop_fm_view.GameView;
import javafx.geometry.Dimension2D;
import javafx.geometry.Point2D;

public class GameModel {
    private final Ball ball;

    private final Dimension2D panelSize;

    private final Paddle paddle;

    private final GameView view;

    private boolean contact = false;

    public GameModel(GameView view) {
        this.view = view;
        this.panelSize = new Dimension2D(view.box.getWidth(), view.box.getHeight());

        double ballRadius = view.ball.getRadius();
        Point2D centerPointBall = new Point2D(view.ball.getCenterX(),
                view.ball.getCenterY());
        this.ball = new Ball(centerPointBall, ballRadius,
                panelSize);

        double x = view.paddle.translateXProperty().doubleValue();
        double y = view.paddle.translateYProperty().doubleValue();
        Point2D centerPointPaddle = new Point2D(x, y);
        Dimension2D paddleDimension = new Dimension2D(view.paddle.getWidth(), view.paddle.getHeight());
        this.paddle = new Paddle(view.box.getWidth(), paddleDimension, centerPointPaddle);
    }

    public void restartGame() {
        resetBall();
        paddle.setPaddle(calculateCenterX(paddle.getWidth()));
    }

    public void resetBall() {
        ball.setCenterPoint(new Point2D(panelSize.getWidth() / 2,
                panelSize.getHeight() / 2));
    }

    private double calculateCenterX(double batWidth) {
        return (panelSize.getWidth() - batWidth) / 2;
    }

    public void contactsPaddle(long time) {
        boolean contactNow = paddle.intersects(ball);
        if (!contact && contactNow) {
            ball.reverseYDirection(time);
            contact = true;
        } else {
            contact = contactNow;
        }
    }

    public Ball getBall() {
        return ball;
    }

    public Paddle getPaddle() {
        return paddle;
    }

    public double getWidth() {
        return panelSize.getWidth();
    }

    public void repaint() {
        view.ball.setTranslateX(ball.getCenterPoint().getX() - ball.getRadius());
        view.ball.setTranslateY(ball.getCenterPoint().getY() - ball.getRadius());

        view.paddle.setTranslateX(paddle.getTranslateX());
        view.paddle.setTranslateY(paddle.getTranslateY());
    }
}
